import * as express from "express";
import * as bodyparser from "body-parser";

import { authRoute } from "../security/token";

import { userFromSession } from "../utils/user";

import { IUser } from "../models/user";

const userRouter = express.Router();
userRouter.use(authRoute);
userRouter.use(bodyparser.json());

// Return the user object if the user is still authenticated
userRouter.get("/me", async (req, res) => {
    //@ts-ignore
    const { db, token } = req;

    const session = await db.collection("sessions").findOne({ token, });
    if (session !== null) {
        const user = await db.collection("users").findOne({ username: session.username });

        // Copy and strip unneeded attributes
        let copy = Object.assign({}, user, {
            sessionToken: token,
        });
        delete copy._id;
        delete copy.hash;
        delete copy.salt;

        res.send({
            error: "0",
            data: copy,
        });
    } else {
        res.send({
            error: "404",
            data: {},
        });
    }
});

// Removes the user's session
userRouter.get("/logout", async (req, res) => {
    // Try to remove the session
    //@ts-ignore
    await req.db.collections("sessions").findOneAndDelete({ session: req.get("Token") });

    res.send({
        error: "0",
        data: {},
    });
});

// TODO: This should be shared with the frontend, to remove code duplication
export enum VocabType {
    NOMEN = 0,
    VERB = 1,
    ADJEKTIV = 2,
    ADVERB = 3,
};

// Gets the user's review queue
userRouter.get("/queue", async (req, res) => {
    console.log("STUB: /user/queue");

    // TODO: Stub
    res.send({
        error: "0",
        data: {
            queue: [{
                german: ["Wein"],
                hint: "Worte auf '-um' sind meistens NeutrUM",
                type: VocabType.NOMEN,
                latin: {
                    grundform: "Vinum",
                    genitiv: "Vini",
                    genus: "Neutrum"
                },
                id: 0
            }]
        },
    });
});

// Get ot set the last review results
userRouter.get("/lastReview", async (req, res) => {
    console.log("STUB(get): /user/lastReview");

    // TODO: Stub
    res.send({
        error: "0",
        data: {
            correct: 6,
            wrong: 2,
        },
    });
});
userRouter.post("/lastReview", async (req, res) => {
    console.log("STUB(post): /user/lastReview");

    // TODO: Stub
    res.send({
        error: "0",
        data: {},
    });
});

// Get the next level
userRouter.get("/nextLevel", async (req, res) => {
    console.log("STUB: /user/nextLevel");

    // TODO: Stub
    res.send({
        error: "0",
        data: {
            name: "Test level",
            desc: "Just a test",
            level: 3,

            done: false,
        },
    });
});

// Mark a level as done
userRouter.post("/level/:id", async (req, res) => {
    console.log("STUB(post): /user/level/:id");

    // TODO: Stub
    res.send({
        error: "0",
        data: {},
    });
});

// Get the data needed for the dashboard 
userRouter.get("/dashboard", async (req, res) => {
    console.log("SEMI-STUB(post): /user/dashboard");

    //@ts-ignore
    const { db, token } = req;

    // Get the user
    const user = await userFromSession(token, db);
    // TODO: if (user)

    const { classId } = user;

    // Fetch the top ten of the class
    const rawTopTen = await db.collection("users").find({
        classId,
    }, {
            sort: {
                score: -1,
            },
            limit: 10,
        }).toArray();

    let nr = 1;
    const topTen = rawTopTen.map((user: IUser) => {
        return {
            username: user.username,
            score: user.score,
            // TODO
            level: 1,
            nr: nr++,
        };
    });

    // TODO: Stub
    res.send({
        error: "200",
        data: {
            // TODO: Get this some way
            nextLevel: {
                name: "Test level",
                desc: "Just a test",
                level: 3,

                done: false,
            },
            topTen,
            lastReview: user.lastReview,
        },
    });
});

export default userRouter;
